@@ -49,6 +49,8 @@ async def fetch_page(
     current_pass: int,
     new_page=1,
     delay_counter=0,
+    *,
+    query_params=None,
 ) -> List[dict]:
     """
     Fetches data from AirCall API
@@ -56,7 +58,12 @@ async def fetch_page(
     dependent on existence of other pages and call limit
     """
     endpoint = f'{BASE_ROUTE}/{dataset}?per_page={PER_PAGE}&page={new_page}'
-    data: dict = await fetch(endpoint, session)
+
+    if query_params:
+        data: dict = await fetch(endpoint, session, query_params)
+    else:
+        data: dict = await fetch(endpoint, session)
+
     logging.getLogger(__file__).info(
         f'Request sent to Aircall for page {new_page} for dataset {dataset}'
     )
@@ -71,7 +78,14 @@ async def fetch_page(
             delay_counter += 1
             logging.getLogger(__file__).info('Retrying Aircall API')
             data_list = await fetch_page(
-                dataset, data_list, session, limit, current_pass, new_page, delay_counter
+                dataset,
+                data_list,
+                session,
+                limit,
+                current_pass,
+                new_page,
+                delay_counter,
+                query_params=query_params,
             )
         else:
             logging.getLogger(__file__).error('Aborting Aircall requests')
@@ -91,21 +105,33 @@ async def fetch_page(
         if next_page_link is not None and current_pass < limit:
             next_page = meta_data['current_page'] + 1
             data_list = await fetch_page(
-                dataset, data_list, session, limit, current_pass, next_page
+                dataset,
+                data_list,
+                session,
+                limit,
+                current_pass,
+                next_page,
+                query_params=query_params,
             )
     else:
         if next_page_link is not None:
             next_page = meta_data['current_page'] + 1
             data_list = await fetch_page(
-                dataset, data_list, session, limit, current_pass, next_page
+                dataset,
+                data_list,
+                session,
+                limit,
+                current_pass,
+                next_page,
+                query_params=query_params,
             )
 
     return data_list
 
 
-async def fetch(new_endpoint, session: ClientSession) -> dict:
+async def fetch(new_endpoint, session: ClientSession, query_params=None) -> dict:
     """The basic fetch function"""
-    async with session.get(new_endpoint) as res:
+    async with session.get(new_endpoint, params=query_params) as res:
         return await res.json()
 
 
@@ -121,6 +147,7 @@ class AircallConnector(ToucanConnector):
     """
 
     _auth_flow = 'oauth2'
+    provided_token: Optional[str]
     auth_flow_id: Optional[str]
     data_source_model: AircallDataSource
 
@@ -147,6 +174,7 @@ class AircallConnector(ToucanConnector):
                 client_secret=kwargs['client_secret'],
             ),
         )
+        self.provided_token = kwargs.get('provided_token', None)
 
     def build_authorization_url(self, **kwargs):
         return self.__dict__['_oauth2_connector'].build_authorization_url(**kwargs)
@@ -160,12 +188,14 @@ class AircallConnector(ToucanConnector):
         return self.__dict__['_oauth2_connector'].retrieve_tokens(authorization_response)
 
     def get_access_token(self):
+        if self.provided_token:
+            return self.provided_token
         return self.__dict__['_oauth2_connector'].get_access_token()
 
-    async def _fetch(self, url, headers=None):
+    async def _fetch(self, url, headers=None, query_params=None):
         """Build the final request along with headers."""
         async with ClientSession(headers=headers) as session:
-            return await fetch(url, session)
+            return await fetch(url, session, query_params=query_params)
 
     def _run_fetch(self, url):
         """Run loop."""
@@ -178,7 +208,9 @@ class AircallConnector(ToucanConnector):
         future = asyncio.ensure_future(self._fetch(url, headers))
         return loop.run_until_complete(future)
 
-    async def _get_data(self, dataset: str, limit) -> Tuple[List[dict], List[dict]]:
+    async def _get_data(
+        self, dataset: str, limit, query_params=None
+    ) -> Tuple[List[dict], List[dict]]:
         """Triggers fetches for data and does preliminary filtering process"""
         access_token = self.get_access_token()
         if not access_token:
@@ -193,14 +225,9 @@ class AircallConnector(ToucanConnector):
                     session,
                     limit,
                     0,
+                    query_params=None,  # for now we don't provide param while querying the teams endpoint
                 ),
-                fetch_page(
-                    dataset,
-                    [],
-                    session,
-                    limit,
-                    0,
-                ),
+                fetch_page(dataset, [], session, limit, 0, query_params=query_params),
             )
             team_response_list = []
             variable_response_list = []
@@ -235,10 +262,10 @@ class AircallConnector(ToucanConnector):
                 tags_data_list += data['tags']
             return tags_data_list
 
-    def run_fetches(self, dataset, limit) -> Tuple[List[dict], List[dict]]:
+    def run_fetches(self, dataset, limit, query_params=None) -> Tuple[List[dict], List[dict]]:
         """sets up event loop and fetches for 'calls' and 'users' datasets"""
         loop = get_loop()
-        future = asyncio.ensure_future(self._get_data(dataset, limit))
+        future = asyncio.ensure_future(self._get_data(dataset, limit, query_params))
         return loop.run_until_complete(future)
 
     def run_fetches_for_tags(self, dataset, limit):
@@ -247,7 +274,7 @@ class AircallConnector(ToucanConnector):
         future = asyncio.ensure_future(self._get_tags(dataset, limit))
         return loop.run_until_complete(future)
 
-    def _retrieve_data(self, data_source: AircallDataSource) -> pd.DataFrame:
+    def _retrieve_data(self, data_source: AircallDataSource, query_params=None) -> pd.DataFrame:
         """retrieves data from AirCall API"""
         dataset = data_source.dataset
         empty_df = build_empty_df(dataset)
@@ -266,7 +293,7 @@ class AircallConnector(ToucanConnector):
             team_data = pd.DataFrame([])
             variable_data = pd.DataFrame([])
             if limit != 0:
-                team_data, variable_data = self.run_fetches(dataset, limit)
+                team_data, variable_data = self.run_fetches(dataset, limit, query_params)
             return build_df(
                 dataset, [empty_df, pd.DataFrame(team_data), pd.DataFrame(variable_data)]
             )
