@@ -2,12 +2,15 @@ import asyncio
 import logging
 import os
 from contextlib import suppress
+from datetime import datetime
 from enum import Enum
 from pathlib import Path
 from typing import List, Optional
 
+import numpy as np
 import pandas as pd
 import requests
+from dateutil import relativedelta
 from pydantic import Field, create_model
 from python_graphql_client import GraphqlClient
 
@@ -51,6 +54,9 @@ TOKEN_URL: str = 'https://github.com/login/oauth/access_token'
 BASE_ROUTE: str = 'https://api.github.com/graphql'
 BASE_ROUTE_REST: str = 'https://api.github.com/'
 NO_CREDENTIALS_ERROR = 'No credentials'
+extraction_start_date = datetime.strftime(
+    datetime.now() - relativedelta.relativedelta(years=1), '%Y-%m-%dT%H:%M:%SZ'
+)
 
 
 class NoCredentialsError(Exception):
@@ -246,6 +252,22 @@ class GithubConnector(ToucanConnector):
                 except ValueError:
                     pass
 
+            # For now we want to retrieve only max 1 year of Pull requests
+            # TODO change this to be able to receive a extraction_start_date date for extraction as a parameter
+            if dataset == GithubDataSet('pull requests'):
+                try:
+                    # Find the first index where PR Creation Date is < extraction_start_date
+                    # Throws IndexError if such index cannot be found
+                    index = np.where(
+                        np.array([pr['PR Creation Date'] for pr in formatted_data])
+                        < extraction_start_date
+                    )[0][0]
+                    formatted_data = formatted_data[:index]
+                    data_list.extend(formatted_data)
+                    return data_list
+                except (IndexError, TypeError):
+                    pass
+
             if dataset == GithubDataSet('pull requests'):
                 data_list.extend(formatted_data)
             else:
@@ -321,7 +343,7 @@ class GithubConnector(ToucanConnector):
                 dataset=dataset,
                 organization=organization,
                 page_limit=page_limit,
-                latest_retrieved_object=latest_retrieved_object[name]
+                latest_retrieved_object=latest_retrieved_object.get(name)
                 if latest_retrieved_object
                 else None,
             )
