@@ -4,7 +4,6 @@ from typing import Optional, Pattern, Union
 
 import pandas as pd
 import pymongo
-from bson.regex import Regex
 from bson.son import SON
 from cached_property import cached_property
 from pydantic import Field, SecretStr, create_model, validator
@@ -258,7 +257,16 @@ class MongoConnector(ToucanConnector):
         # We simply append the match regex at the end of the query,
         # Mongo will then optimize the pipeline to move the match regex to its most convenient position
         # (c.f https://docs.mongodb.com/manual/core/aggregation-pipeline-optimization/#pipeline-sequence-optimization)
-        data_source.query.append({'$match': {field: {'$regex': Regex.from_native(regex)}}})
+        # Since Mongo '$regex' operator doesn't work with integer values, we need to check the stringified versions
+        regex_step = {
+            '$expr': {
+                '$regexMatch': {
+                    'input': {'$toString': f'${field}'},
+                    'regex': regex.pattern,
+                }
+            }
+        }
+        data_source.query.append({'$match': regex_step})
         return self.get_slice(data_source, permissions, limit=limit).df
 
     @decorate_func_with_retry
